@@ -1,6 +1,6 @@
 package main
 
-import "./namespace_virtual_method"
+import "swigtests/namespace_virtual_method"
 
 func main() {
 	_ = namespace_virtual_method.NewSpam()
