@@ -434,9 +434,6 @@ if v != 4:
 # Check the bounds for converting various types
 #
 
-# ctypes not available until 2.5
-if sys.version_info[0:2] <= (2, 4):
-    sys.exit(0)
 import ctypes
 
 # Get the minimum and maximum values that fit in signed char, short, int, long, and long long
@@ -593,6 +590,5 @@ def checkType(name, maxfunc, maxval, minfunc, minval, echofunc):
         raise RuntimeError("bad " + name + " typemap")
 
 # sys.maxsize is the largest value supported by Py_ssize_t, which should be the same as ptrdiff_t
-if sys.version_info[0:2] >= (2, 6):
-    checkType("ptrdiff_t", get_ptrdiff_max, sys.maxsize,           get_ptrdiff_min, -(sys.maxsize + 1), ptrdiff_echo)
-    checkType("size_t",    get_size_max,    (2 * sys.maxsize) + 1, get_size_min,    0,                  size_echo)
+checkType("ptrdiff_t", get_ptrdiff_max, sys.maxsize,           get_ptrdiff_min, -(sys.maxsize + 1), ptrdiff_echo)
+checkType("size_t",    get_size_max,    (2 * sys.maxsize) + 1, get_size_min,    0,                  size_echo)
