@@ -1,4 +1,6 @@
-(define x "hello")
+; The test string has some non-ascii characters added
+; because our guile wrappers had bugs in that area
+(define x "hello - æææ")
 
 (if (not (string=? (test-value x) x))
   (begin (error "Error 1") (exit 1)))
@@ -25,10 +27,10 @@
 
 ;; MemberString should be a wrapped class
 (define scl (Structure-MemberString-get struct))
-(if (not (string=? (std-string-str-get scl) ""))
+(if (not (string=? scl ""))
   (error "Error 4.5"))
-(std-string-str-set scl "and how")
-(if (not (string=? (std-string-str-get (Structure-MemberString-get struct)) "and how"))
+(Structure-MemberString-set struct "and how")
+(if (not (string=? (Structure-MemberString-get struct) "and how"))
   (error "Error 5"))
 (if (not (string=? (Structure-MemberString2-get struct) "member string 2"))
   (error "Error 6"))
@@ -43,4 +45,20 @@
 (if (not (string=? (Structure-ConstStaticMemberString) "const static member string"))
   (error "Error 10"))
 
+(if (not (string=? (stdstring-empty) ""))
+  (error "stdstring-empty test"))
+(if (not (string=? (c-empty) ""))
+  (error "c-empty test"))
+; C++ NULL is mapped to #f (false) here rather than null.  I don't know guile
+; enough to know if that makes sense or not.
+(if (c-null)
+  (error "c-null test"))
+; FIXME: However, #f doesn't round-trip, so something seems wrong.
+; (if (get-null (c-null))
+;   (error "get-null c-empty test"))
+(if (not (string=? (get-null (c-empty)) "non-null"))
+  (error "get-null c-empty test"))
+(if (not (string=? (get-null (stdstring-empty)) "non-null"))
+  (error "get-null stdstring-empty test"))
+
 (exit 0)
