@@ -4,15 +4,13 @@
  * terms also apply to certain portions of SWIG. The full details of the SWIG
  * license and copyrights can be found in the LICENSE and COPYRIGHT files
  * included with the SWIG source code as distributed by the SWIG developers
- * and at http://www.swig.org/legal.html.
+ * and at https://www.swig.org/legal.html.
  *
  * hash.c
  *
  *     Implements a simple hash table object.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_hash_c[] = "$Id: hash.c 12221 2010-09-15 20:17:11Z wsfulton $";
-
 #include "dohint.h"
 
 extern DohObjInfo DohHashType;
@@ -175,10 +173,7 @@ static void resize(Hash *h) {
     p = p + 2;
   }
 
-  table = (HashNode **) DohMalloc(newsize * sizeof(HashNode *));
-  for (i = 0; i < newsize; i++) {
-    table[i] = 0;
-  }
+  table = (HashNode **) DohCalloc(newsize, sizeof(HashNode *));
 
   /* Walk down the old set of nodes and re-place */
   h->hashsize = newsize;
@@ -363,7 +358,7 @@ static DohIterator Hash_nextiter(DohIterator iter) {
 }
 
 /* -----------------------------------------------------------------------------
- * Hash_keys(DOH *)
+ * Hash_keys()
  *
  * Return a list of keys
  * ----------------------------------------------------------------------------- */
@@ -415,12 +410,12 @@ static DOH *Hash_str(DOH *ho) {
 
   s = NewStringEmpty();
   if (ObjGetMark(ho)) {
-    Printf(s, "Hash(0x%x)", ho);
+    Printf(s, "Hash(%p)", ho);
     return s;
   }
   if (expanded >= max_expand) {
     /* replace each hash attribute with a '.' */
-    Printf(s, "Hash(0x%x) {", ho);
+    Printf(s, "Hash(%p) {", ho);
     for (i = 0; i < h->hashsize; i++) {
       n = h->hashtable[i];
       while (n) {
@@ -432,7 +427,7 @@ static DOH *Hash_str(DOH *ho) {
     return s;
   }
   ObjSetMark(ho, 1);
-  Printf(s, "Hash(0x%x) {\n", ho);
+  Printf(s, "Hash(%p) {\n", ho);
   for (i = 0; i < h->hashsize; i++) {
     n = h->hashtable[i];
     while (n) {
