@@ -1,17 +1,17 @@
 /* ----------------------------------------------------------------------------- 
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at https://www.swig.org/legal.html.
+ *
  * void.c
  *
  *     Implements a "void" object that is really just a DOH container around
  *     an arbitrary C object represented as a void *.
- * 
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1999-2000.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_void_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/void.c,v 1.4 2006/11/01 23:54:50 wsfulton Exp $";
-
 #include "dohint.h"
 
 typedef struct {
