@@ -4,7 +4,7 @@
  * terms also apply to certain portions of SWIG. The full details of the SWIG
  * license and copyrights can be found in the LICENSE and COPYRIGHT files
  * included with the SWIG source code as distributed by the SWIG developers
- * and at http://www.swig.org/legal.html.
+ * and at https://www.swig.org/legal.html.
  *
  * interface.cxx
  *
@@ -15,6 +15,7 @@
  * ----------------------------------------------------------------------------- */
 
 #include "swigmod.h"
+#include "cparse.h"
 
 static bool interface_feature_enabled = false;
 
@@ -28,25 +29,31 @@ static bool interface_feature_enabled = false;
 
 static List *collect_interface_methods(Node *n) {
   List *methods = NewList();
-  if (Hash *bases = Getattr(n, "interface:bases")) {
-    List *keys = Keys(bases);
-    for (Iterator base = First(keys); base.item; base = Next(base)) {
-      Node *cls = Getattr(bases, base.item);
+  if (List *bases = Getattr(n, "interface:bases")) {
+    for (Iterator base = First(bases); base.item; base = Next(base)) {
+      Node *cls = base.item;
       if (cls == n)
 	continue;
       for (Node *child = firstChild(cls); child; child = nextSibling(child)) {
 	if (Cmp(nodeType(child), "cdecl") == 0) {
 	  if (GetFlag(child, "feature:ignore") || Getattr(child, "interface:owner"))
 	    continue; // skip methods propagated to bases
-	  Node *m = Copy(child);
-	  set_nextSibling(m, NIL);
-	  set_previousSibling(m, NIL);
-	  Setattr(m, "interface:owner", cls);
-	  Append(methods, m);
+	  if (!checkAttribute(child, "kind", "function"))
+	    continue;
+	  if (checkAttribute(child, "storage", "static"))
+	    continue; // accept virtual methods, non-virtual methods too... mmm??. Warn that the interface class has something that is not a virtual method?
+	  Node *nn = copyNode(child);
+	  Setattr(nn, "interface:owner", cls);
+	  ParmList *parms = CopyParmList(Getattr(child, "parms"));
+	  Setattr(nn, "parms", parms);
+	  Delete(parms);
+	  ParmList *throw_parm_list = Getattr(child, "throws");
+	  if (throw_parm_list)
+	    Setattr(nn, "throws", CopyParmList(throw_parm_list));
+	  Append(methods, nn);
 	}
       }
     }
-    Delete(keys);
   }
   return methods;
 }
@@ -55,17 +62,17 @@ static List *collect_interface_methods(Node *n) {
  * collect_interface_bases
  * ----------------------------------------------------------------------------- */
 
-static void collect_interface_bases(Hash *bases, Node *n) {
-  if (Getattr(n, "feature:interface")) {
+static void collect_interface_bases(List *bases, Node *n) {
+  if (GetFlag(n, "feature:interface")) {
     String *name = Getattr(n, "interface:name");
     if (!Getattr(bases, name))
-      Setattr(bases, name, n);
+      Append(bases, n);
   }
 
   if (List *baselist = Getattr(n, "bases")) {
     for (Iterator base = First(baselist); base.item; base = Next(base)) {
       if (!GetFlag(base.item, "feature:ignore")) {
-	if (Getattr(base.item, "feature:interface"))
+	if (GetFlag(base.item, "feature:interface"))
 	  collect_interface_bases(bases, base.item);
       }
     }
@@ -83,21 +90,21 @@ static void collect_interface_bases(Hash *bases, Node *n) {
  * ----------------------------------------------------------------------------- */
 
 static void collect_interface_base_classes(Node *n) {
-  if (Getattr(n, "feature:interface")) {
+  if (GetFlag(n, "feature:interface")) {
     // check all bases are also interfaces
     if (List *baselist = Getattr(n, "bases")) {
       for (Iterator base = First(baselist); base.item; base = Next(base)) {
 	if (!GetFlag(base.item, "feature:ignore")) {
-	  if (!Getattr(base.item, "feature:interface")) {
+	  if (!GetFlag(base.item, "feature:interface")) {
 	    Swig_error(Getfile(n), Getline(n), "Base class '%s' of '%s' is not similarly marked as an interface.\n", SwigType_namestr(Getattr(base.item, "name")), SwigType_namestr(Getattr(n, "name")));
-	    SWIG_exit(EXIT_FAILURE);
+	    Exit(EXIT_FAILURE);
 	  }
 	}
       }
     }
   }
 
-  Hash *interface_bases = NewHash();
+  List *interface_bases = NewList();
   collect_interface_bases(interface_bases, n);
   if (Len(interface_bases) == 0)
     Delete(interface_bases);
@@ -110,11 +117,11 @@ static void collect_interface_base_classes(Node *n) {
  * ----------------------------------------------------------------------------- */
 
 static void process_interface_name(Node *n) {
-  if (Getattr(n, "feature:interface")) {
+  if (GetFlag(n, "feature:interface")) {
     String *interface_name = Getattr(n, "feature:interface:name");
     if (!Len(interface_name)) {
       Swig_error(Getfile(n), Getline(n), "The interface feature for '%s' is missing the name attribute.\n", SwigType_namestr(Getattr(n, "name")));
-      SWIG_exit(EXIT_FAILURE);
+      Exit(EXIT_FAILURE);
     }
     if (Strchr(interface_name, '%')) {
       String *name = NewStringf(interface_name, Getattr(n, "sym:name"));
@@ -139,7 +146,7 @@ void Swig_interface_propagate_methods(Node *n) {
     process_interface_name(n);
     collect_interface_base_classes(n);
     List *methods = collect_interface_methods(n);
-    bool is_interface = Getattr(n, "feature:interface") != 0;
+    bool is_interface = GetFlag(n, "feature:interface") ? true : false;
     for (Iterator mi = First(methods); mi.item; mi = Next(mi)) {
       if (!is_interface && GetFlag(mi.item, "abstract"))
 	continue;
@@ -164,8 +171,26 @@ void Swig_interface_propagate_methods(Node *n) {
       }
       Delete(this_decl_resolved);
       if (!identically_overloaded_method) {
-	// TODO: Fix if the method is overloaded with different arguments / has default args
-	appendChild(n, mi.item);
+	// Add method copied from base class to this derived class
+	Node *cn = mi.item;
+	Delattr(cn, "sym:overname");
+	String *prefix = Getattr(n, "name");
+	String *name = Getattr(cn, "name");
+	String *decl = Getattr(cn, "decl");
+	String *oldname = Getattr(cn, "sym:name");
+
+	String *symname = Swig_name_make(cn, prefix, name, decl, oldname);
+	if (Strcmp(symname, "$ignore") != 0) {
+	  Symtab *oldscope = Swig_symbol_setscope(Getattr(n, "symtab"));
+	  Node *on = Swig_symbol_add(symname, cn);
+	  (void)on;
+	  assert(on == cn);
+
+	  // Features from the copied base class method are already present, now add in features specific to the added method in the derived class
+	  Swig_features_get(Swig_cparse_features(), Swig_symbol_qualifiedscopename(0), name, decl, cn);
+	  Swig_symbol_setscope(oldscope);
+	  appendChild(n, cn);
+	}
       } else {
 	Delete(mi.item);
       }
