@@ -69,6 +69,8 @@ static Hash *typemaps;
  * resolving the template parameters.
  *
  * This is a copy and modification of feature_identifier_fix in parser.y.
+ * Also, Swig_smartptr_upcast() could be removed if SwigType_typedef_resolve_all
+ * is fixed to resolve all template parameters like below.
  * ----------------------------------------------------------------------------- */
 
 static SwigType *typemap_identifier_fix(const SwigType *s) {
@@ -102,6 +104,7 @@ static Hash *get_typemap(const SwigType *type) {
     dtype = Swig_symbol_type_qualify(ty, 0);
     type = dtype;
     Delete(ty);
+    Delete(rty);
   }
 
   /* remove unary scope operator (::) prefix indicating global scope for looking up in the hashmap */
@@ -865,13 +868,6 @@ static Hash *typemap_search_multi(const_String_or_char_ptr tmap_method, ParmList
 }
 
 
-/* -----------------------------------------------------------------------------
- * typemap_replace_vars()
- *
- * Replaces typemap variables on a string.  index is the $n variable.
- * type and pname are the type and parameter name.
- * ----------------------------------------------------------------------------- */
-
 static void replace_local_types(ParmList *p, const String *name, const String *rep) {
   SwigType *t;
   while (p) {
@@ -891,7 +887,23 @@ static int check_locals(ParmList *p, const char *s) {
   return 0;
 }
 
-static int typemap_replace_vars(String *s, ParmList *locals, SwigType *type, SwigType *rtype, String *pname, String *lname, int index) {
+static int replace_with_override(String *src, const DOHString_or_char *token, const DOHString_or_char *rep, int flags, Hash *override_vars) {
+  String *override_replace = override_vars ? Getattr(override_vars, token) : NULL;
+  if (override_replace)
+    return Replace(src, token, override_replace, flags);
+  return Replace(src, token, rep, flags);
+}
+
+/* -----------------------------------------------------------------------------
+ * typemap_replace_vars()
+ *
+ * Replaces typemap variables in a string.  index is the $n variable.
+ * type and pname are the type and parameter name.
+ * override_vars are only used to replace variable names (NOT types), so is only
+ * used for $n, but not the various ways of representing types: $n_ltype, $n_type etc
+ * ----------------------------------------------------------------------------- */
+
+static int typemap_replace_vars(String *s, ParmList *locals, SwigType *type, SwigType *rtype, String *pname, String *lname, int index, Hash *override_vars) {
   char var[512];
   char *varname;
   SwigType *ftype;
@@ -1166,7 +1178,7 @@ static int typemap_replace_vars(String *s, ParmList *locals, SwigType *type, Swi
     Delete(lex_type);
   }
 
-  /* Replace any $n. with (&n)-> */
+  /* Replace variable usage $n. with (&$n)-> */
   {
     char temp[64];
     sprintf(var, "$%d.", index);
@@ -1176,7 +1188,7 @@ static int typemap_replace_vars(String *s, ParmList *locals, SwigType *type, Swi
 
   /* Replace the bare $n variable */
   sprintf(var, "$%d", index);
-  bare_substitution_count = Replace(s, var, lname, DOH_REPLACE_NUMBER_END);
+  bare_substitution_count = replace_with_override(s, var, lname, DOH_REPLACE_NUMBER_END, override_vars);
   Delete(ftype);
   return bare_substitution_count;
 }
@@ -1290,7 +1302,7 @@ static void typemap_merge_fragment_kwargs(Parm *kw) {
 	reattach_kw = prev_kw;
       }
       if (kwtype) {
-        String *mangle = Swig_string_mangle(kwtype);
+        String *mangle = Swig_name_mangle_type(kwtype);
         Append(fragment, mangle);
         Delete(mangle);
         /* Remove 'type' from kwargs so it's not duplicated later */
@@ -1498,7 +1510,7 @@ static String *Swig_typemap_lookup_impl(const_String_or_char_ptr tmap_method, No
   }
 
   matchtype = mtype && SwigType_isarray(mtype) ? mtype : type;
-  num_substitutions = typemap_replace_vars(s, locals, matchtype, type, pname, (char *) lname, 1);
+  num_substitutions = typemap_replace_vars(s, locals, matchtype, type, pname, (char *) lname, 1, NULL);
   if (optimal_substitution && num_substitutions > 1) {
     Swig_warning(WARN_TYPEMAP_OUT_OPTIMAL_MULTIPLE, Getfile(node), Getline(node), "Multiple calls to %s might be generated due to\n", Swig_name_decl(node));
     Swig_warning(WARN_TYPEMAP_OUT_OPTIMAL_MULTIPLE, Getfile(s), Getline(s), "optimal attribute usage in the out typemap.\n");
@@ -1529,7 +1541,7 @@ static String *Swig_typemap_lookup_impl(const_String_or_char_ptr tmap_method, No
       SwigType *mtype = Getattr(node, "tmap:match");
       SwigType *matchtype = mtype ? mtype : ptype;
       ParmList *parm_sublist;
-      typemap_replace_vars(value, NULL, matchtype, ptype, pname, (char *)lname, 1);
+      typemap_replace_vars(value, NULL, matchtype, ptype, pname, (char *)lname, 1, NULL);
 
       /* Expand special variable macros (embedded typemaps) in typemap attributes. */
       parm_sublist = NewParmWithoutFileLineInfo(ptype, pname);
@@ -1538,7 +1550,7 @@ static String *Swig_typemap_lookup_impl(const_String_or_char_ptr tmap_method, No
       Delete(parm_sublist);
     }
     if (kwtype) {
-      String *mangle = Swig_string_mangle(kwtype);
+      String *mangle = Swig_name_mangle_type(kwtype);
       Append(value, mangle);
       Delete(mangle);
     }
@@ -1572,7 +1584,7 @@ static String *Swig_typemap_lookup_impl(const_String_or_char_ptr tmap_method, No
   /* Print warnings, if any */
   warning = typemap_warn(cmethod, node);
   if (warning) {
-    typemap_replace_vars(warning, 0, matchtype, type, pname, (char *) lname, 1);
+    typemap_replace_vars(warning, 0, matchtype, type, pname, (char *) lname, 1, NULL);
     Replace(warning, "$name", pname, DOH_REPLACE_ANY);
     if (symname)
       Replace(warning, "$symname", symname, DOH_REPLACE_ANY);
@@ -1645,7 +1657,7 @@ static void typemap_attach_kwargs(Hash *tm, const_String_or_char_ptr tmap_method
       String *lname = Getattr(p, "lname");
       SwigType *mtype = Getattr(p, "tmap:match");
       SwigType *matchtype = mtype ? mtype : type;
-      typemap_replace_vars(value, NULL, matchtype, type, pname, lname, i + 1);
+      typemap_replace_vars(value, NULL, matchtype, type, pname, lname, i + 1, NULL);
       p = nextSibling(p);
     }
 
@@ -1717,12 +1729,13 @@ static String *typemap_get_option(Hash *tm, const_String_or_char_ptr name) {
  * code can be retrieved for the first parameter with a call to Getattr(parm, "tmap:in")
  * and the "numinputs" attribute can be retrieved with a call to Getattr(parm, "tmap:in:numinputs").
  *
- * tmap_method - typemap method, eg "in", "out", "newfree"
- * parms       - parameter list to attach each typemap and all typemap attributes
- * f           - wrapper code to generate into if non null
+ * tmap_method   - typemap method, eg "in", "out", "newfree"
+ * parms         - parameter list to attach each typemap and all typemap attributes
+ * f             - wrapper code to generate into if non null
+ * override_vars - Hash of variables from $typemap(...) that will override special variable replacements
  * ----------------------------------------------------------------------------- */
 
-void Swig_typemap_attach_parms(const_String_or_char_ptr tmap_method, ParmList *parms, Wrapper *f) {
+static void typemap_attach_parms(const_String_or_char_ptr tmap_method, ParmList *parms, Wrapper *f, Hash *override_vars) {
   Parm *p, *firstp;
   Hash *tm;
   int nmatch = 0;
@@ -1838,7 +1851,7 @@ void Swig_typemap_attach_parms(const_String_or_char_ptr tmap_method, ParmList *p
       SwigType *mtype = Getattr(p, "tmap:match");
       SwigType *matchtype = mtype ? mtype : type;
 
-      typemap_replace_vars(s, locals, matchtype, type, pname, lname, i + 1);
+      typemap_replace_vars(s, locals, matchtype, type, pname, lname, i + 1, override_vars);
       if (mtype)
 	Delattr(p, "tmap:match");
 
@@ -1886,7 +1899,7 @@ void Swig_typemap_attach_parms(const_String_or_char_ptr tmap_method, ParmList *p
       String *lname = Getattr(firstp, "lname");
       SwigType *mtype = Getattr(firstp, "tmap:match");
       SwigType *matchtype = mtype ? mtype : type;
-      typemap_replace_vars(warning, 0, matchtype, type, pname, lname, 1);
+      typemap_replace_vars(warning, 0, matchtype, type, pname, lname, 1, override_vars);
       Replace(warning, "$argnum", temp, DOH_REPLACE_ANY);
       Swig_warning(0, Getfile(firstp), Getline(firstp), "%s\n", warning);
       Delete(warning);
@@ -1909,6 +1922,10 @@ void Swig_typemap_attach_parms(const_String_or_char_ptr tmap_method, ParmList *p
 
 }
 
+void Swig_typemap_attach_parms(const_String_or_char_ptr tmap_method, ParmList *parms, Wrapper *f) {
+  typemap_attach_parms(tmap_method, parms, f, NULL);
+}
+
 /* Splits the arguments of an embedded typemap */
 static List *split_embedded_typemap(String *s) {
   List *args = 0;
@@ -1953,7 +1970,7 @@ static List *split_embedded_typemap(String *s) {
       level--;
     if (*c == '<')
       angle_level++;
-    if (*c == '>')
+    if (*c == '>' && *(c - 1) != '-')
       angle_level--;
     if (isspace((int) *c) && leading)
       start++;
@@ -1987,7 +2004,9 @@ void Swig_typemap_replace_embedded_typemap(String *s, Node *file_line_node) {
  *
  *   $typemap(method, typelist, var1=value, var2=value, ...)
  *
- * where varx parameters are optional and undocumented; they were used in an earlier version of $typemap.
+ * where varx parameters are optional and undocumented; they were initially an experiment in the first implementation of $typemap.
+ * They allow one to override known special variable replacements or even add in new special variables for replacement
+ * from the calling typemap to the called typemap.
  * A search is made using the typemap matching rules of form:
  *
  *   %typemap(method) typelist {...}
@@ -2032,7 +2051,7 @@ static void replace_embedded_typemap(String *s, ParmList *parm_sublist, Wrapper
     if (!syntax_error) {
       List *l;
       String *tmap_method;
-      Hash *vars;
+      Hash *override_vars;
       syntax_error = 1;
 
       /* Split apart each parameter in $typemap(...) */
@@ -2061,8 +2080,8 @@ static void replace_embedded_typemap(String *s, ParmList *parm_sublist, Wrapper
 	  Delete(empty_string);
 	}
 
-	/* process optional extra parameters - the variable replacements (undocumented) */
-	vars = NewHash();
+	/* process optional extra parameters - the override variable replacements (undocumented) */
+	override_vars = NewHash();
 	{
 	  int i, ilen;
 	  ilen = Len(l);
@@ -2074,7 +2093,7 @@ static void replace_embedded_typemap(String *s, ParmList *parm_sublist, Wrapper
 	      String *name = NewStringWithSize(c, (int)(eq - c));
 	      String *value = NewString(eq + 1);
 	      Insert(name, 0, "$");
-	      Setattr(vars, name, value);
+	      Setattr(override_vars, name, value);
 	    } else {
 	      to_match_parms = 0; /* error - variable replacement parameters must be of form varname=value */
 	    }
@@ -2103,10 +2122,10 @@ static void replace_embedded_typemap(String *s, ParmList *parm_sublist, Wrapper
 	    if (found_colon) {
 	      /* Substitute from a keyword argument to a typemap. Avoid emitting local variables from the attached typemap by passing NULL for the file. */
 	      String *temp_tmap_method = NewStringWithSize(Char(tmap_method), (int)(found_colon - Char(tmap_method)));
-	      Swig_typemap_attach_parms(temp_tmap_method, to_match_parms, NULL);
+	      typemap_attach_parms(temp_tmap_method, to_match_parms, NULL, override_vars);
 	      Delete(temp_tmap_method);
 	    } else {
-	      Swig_typemap_attach_parms(tmap_method, to_match_parms, f);
+	      typemap_attach_parms(tmap_method, to_match_parms, f, override_vars);
 	    }
 	    already_substituting--;
 
@@ -2119,7 +2138,7 @@ static void replace_embedded_typemap(String *s, ParmList *parm_sublist, Wrapper
 	      if (!Getattr(to_match_parms, attr)) {
 		/* Replace parameter variables */
 		Iterator ki;
-		for (ki = First(vars); ki.key; ki = Next(ki)) {
+		for (ki = First(override_vars); ki.key; ki = Next(ki)) {
 		  Replace(tm, ki.key, ki.item, DOH_REPLACE_ANY);
 		}
 		/* offer the target language module the chance to make special variable substitutions */
@@ -2148,7 +2167,7 @@ static void replace_embedded_typemap(String *s, ParmList *parm_sublist, Wrapper
 	  }
 	  syntax_error = 0;
 	}
-	Delete(vars);
+	Delete(override_vars);
       }
       Delete(l);
     }
